@@ -11,34 +11,30 @@ enabled.
 """
 from urllib2 import Request, urlopen, HTTPError
 from urllib import urlencode
-from urlparse import urlsplit
 
 from openid.consumer.consumer import Consumer, SUCCESS, CANCEL, FAILURE
 from openid.consumer.discover import DiscoveryFailure
 from openid.extensions import sreg, ax
 
-from oauth2 import Consumer as OAuthConsumer, Token, Request as OAuthRequest, \
-                   SignatureMethod_HMAC_SHA1
+from oauth2 import Consumer as OAuthConsumer, Token, Request as OAuthRequest
 
-from django.db import models
 from django.contrib.auth import authenticate
 from django.contrib.auth.backends import ModelBackend
 from django.utils import simplejson
 from django.utils.importlib import import_module
 
+from social_auth.models import UserSocialAuth
 from social_auth.utils import setting, log, model_to_ctype, ctype_to_model, \
-                              clean_partial_pipeline
+                              clean_partial_pipeline, url_add_parameters, \
+                              get_random_string, constant_time_compare
 from social_auth.store import DjangoOpenIDStore
 from social_auth.backends.exceptions import StopPipeline, AuthException, \
                                             AuthFailed, AuthCanceled, \
                                             AuthUnknownError, AuthTokenError, \
-                                            AuthMissingParameter
-
-
-if setting('SOCIAL_AUTH_USER_MODEL'):
-    User = models.get_model(*setting('SOCIAL_AUTH_USER_MODEL').rsplit('.', 1))
-else:
-    from django.contrib.auth.models import User
+                                            AuthMissingParameter, \
+                                            AuthStateMissing, \
+                                            AuthStateForbidden
+from social_auth.backends.utils import build_consumer_oauth_request
 
 
 # OpenID configuration
@@ -127,7 +123,13 @@ class SocialAuthBackend(ModelBackend):
         """Pipeline"""
         out = kwargs.copy()
 
-        for name in pipeline:
+        if 'pipeline_index' in kwargs:
+            base_index = int(kwargs['pipeline_index'])
+        else:
+            base_index = 0
+
+        for idx, name in enumerate(pipeline):
+            out['pipeline_index'] = base_index + idx
             mod_name, func_name = name.rsplit('.', 1)
             try:
                 mod = import_module(mod_name)
@@ -188,10 +190,7 @@ class SocialAuthBackend(ModelBackend):
         """
         Return user with given ID from the User model used by this backend
         """
-        try:
-            return User.objects.get(pk=user_id)
-        except User.DoesNotExist:
-            return None
+        return UserSocialAuth.get_user(user_id)
 
 
 class OAuthBackend(SocialAuthBackend):
@@ -320,9 +319,8 @@ class BaseAuth(object):
     """Base authentication class, new authenticators should subclass
     and implement needed methods.
 
-        @AUTH_BACKEND   Authorization backend related with this service
+        AUTH_BACKEND   Authorization backend related with this service
     """
-
     AUTH_BACKEND = None
 
     def __init__(self, request, redirect):
@@ -406,9 +404,11 @@ class BaseAuth(object):
         Override if extra operations are needed.
         """
         if association_id:
-            user.social_auth.get(id=association_id).delete()
+            UserSocialAuth.get_social_auth_for_user(user)\
+                            .get(id=association_id).delete()
         else:
-            user.social_auth.filter(provider=self.AUTH_BACKEND.name).delete()
+            UserSocialAuth.get_social_auth_for_user(user)\
+                            .filter(provider=self.AUTH_BACKEND.name).delete()
 
     def build_absolute_uri(self, path=None):
         """Build absolute URI for given path. Replace http:// schema with
@@ -505,13 +505,9 @@ class OpenIdAuth(BaseAuth):
 
     def openid_request(self, extra_params=None):
         """Return openid request"""
-        openid_url = self.openid_url()
-        if extra_params:
-            query = urlsplit(openid_url).query
-            openid_url += (query and '&' or '?') + urlencode(extra_params)
-
         try:
-            return self.consumer().begin(openid_url)
+            return self.consumer().begin(url_add_parameters(self.openid_url(),
+                                                            extra_params))
         except DiscoveryFailure, err:
             raise AuthException(self, 'OpenID discovery error: %s' % err)
 
@@ -534,12 +530,13 @@ class BaseOAuth(BaseAuth):
         super(BaseOAuth, self).__init__(request, redirect)
         self.redirect_uri = self.build_absolute_uri(self.redirect)
 
-    def get_key_and_secret(self):
+    @classmethod
+    def get_key_and_secret(cls):
         """Return tuple with Consumer Key and Consumer Secret for current
         service provider. Must return (key, secret), order *must* be respected.
         """
-        return setting(self.SETTINGS_KEY_NAME), \
-               setting(self.SETTINGS_SECRET_NAME)
+        return setting(cls.SETTINGS_KEY_NAME), \
+               setting(cls.SETTINGS_SECRET_NAME)
 
     @classmethod
     def enabled(cls):
@@ -552,10 +549,10 @@ class ConsumerBasedOAuth(BaseOAuth):
     """Consumer based mechanism OAuth authentication, fill the needed
     parameters to communicate properly with authentication service.
 
-        @AUTHORIZATION_URL       Authorization service url
-        @REQUEST_TOKEN_URL       Request token URL
-        @ACCESS_TOKEN_URL        Access token URL
-        @SERVER_URL              Authorization server URL
+        AUTHORIZATION_URL       Authorization service url
+        REQUEST_TOKEN_URL       Request token URL
+        ACCESS_TOKEN_URL        Access token URL
+        SERVER_URL              Authorization server URL
     """
     AUTHORIZATION_URL = ''
     REQUEST_TOKEN_URL = ''
@@ -615,18 +612,10 @@ class ConsumerBasedOAuth(BaseOAuth):
 
     def oauth_request(self, token, url, extra_params=None):
         """Generate OAuth request, setups callback url"""
-        params = {'oauth_callback': self.redirect_uri}
-        if extra_params:
-            params.update(extra_params)
-
-        if 'oauth_verifier' in self.data:
-            params['oauth_verifier'] = self.data['oauth_verifier']
-        request = OAuthRequest.from_consumer_and_token(self.consumer,
-                                                       token=token,
-                                                       http_url=url,
-                                                       parameters=params)
-        request.sign_request(SignatureMethod_HMAC_SHA1(), self.consumer, token)
-        return request
+        return build_consumer_oauth_request(self, token, url,
+                                            self.redirect_uri,
+                                            self.data.get('oauth_verifier'),
+                                            extra_params)
 
     def fetch_response(self, request):
         """Executes request and fetchs service response"""
@@ -655,18 +644,42 @@ class BaseOAuth2(BaseOAuth):
         http://tools.ietf.org/html/draft-ietf-oauth-v2-10
 
     Attributes:
-        @AUTHORIZATION_URL       Authorization service url
-        @ACCESS_TOKEN_URL        Token URL
+        AUTHORIZATION_URL       Authorization service url
+        ACCESS_TOKEN_URL        Token URL
     """
     AUTHORIZATION_URL = None
     ACCESS_TOKEN_URL = None
     SCOPE_SEPARATOR = ' '
     RESPONSE_TYPE = 'code'
+    SCOPE_VAR_NAME = None
+    DEFAULT_SCOPE = None
+    REDIRECT_STATE = True
+
+    def state_token(self):
+        """Generate csrf token to include as state parameter."""
+        return get_random_string(32)
+
+    def get_redirect_uri(self, state):
+        """Build redirect_uri with redirect_state parameter."""
+        uri = self.redirect_uri
+        if self.REDIRECT_STATE:
+            uri = url_add_parameters(uri, {'redirect_state': state})
+        return uri
 
     def auth_url(self):
         """Return redirect url"""
         client_id, client_secret = self.get_key_and_secret()
-        args = {'client_id': client_id, 'redirect_uri': self.redirect_uri}
+        state = self.state_token()
+        # Store state in session for further request validation. The state
+        # value is passed as state parameter (as specified in OAuth2 spec), but
+        # also added to redirect_uri, that way we can still verify the request
+        # if the provider doesn't implement the state parameter.
+        self.request.session[self.AUTH_BACKEND.name + '_state'] = state
+        args = {
+            'client_id': client_id,
+            'state': state,
+            'redirect_uri': self.get_redirect_uri(state)
+        }
 
         scope = self.get_scope()
         if scope:
@@ -677,18 +690,35 @@ class BaseOAuth2(BaseOAuth):
         args.update(self.auth_extra_arguments())
         return self.AUTHORIZATION_URL + '?' + urlencode(args)
 
+    def validate_state(self):
+        """Validate state value. Raises exception on error, returns state
+        value if valid."""
+        state = self.request.session.get(self.AUTH_BACKEND.name + '_state')
+        request_state = self.data.get('state') or \
+                        self.data.get('redirect_state')
+        if not request_state:
+            raise AuthMissingParameter(self, 'state')
+        elif not state:
+            raise AuthStateMissing(self, 'state')
+        elif not constant_time_compare(request_state, state):
+            raise AuthStateForbidden(self)
+        return state
+
     def auth_complete(self, *args, **kwargs):
         """Completes loging process, must return user instance"""
         if self.data.get('error'):
             error = self.data.get('error_description') or self.data['error']
             raise AuthFailed(self, error)
 
+        state = self.validate_state()
         client_id, client_secret = self.get_key_and_secret()
-        params = {'grant_type': 'authorization_code',  # request auth code
-                  'code': self.data.get('code', ''),  # server response code
-                  'client_id': client_id,
-                  'client_secret': client_secret,
-                  'redirect_uri': self.redirect_uri}
+        params = {
+            'grant_type': 'authorization_code',  # request auth code
+            'code': self.data.get('code', ''),  # server response code
+            'client_id': client_id,
+            'client_secret': client_secret,
+            'redirect_uri': self.get_redirect_uri(state)
+        }
         headers = {'Content-Type': 'application/x-www-form-urlencoded',
                     'Accept': 'application/json'}
         request = Request(self.ACCESS_TOKEN_URL, data=urlencode(params),
@@ -719,7 +749,10 @@ class BaseOAuth2(BaseOAuth):
 
     def get_scope(self):
         """Return list with needed access scope"""
-        return []
+        scope = self.DEFAULT_SCOPE or []
+        if self.SCOPE_VAR_NAME:
+            scope = scope + setting(self.SCOPE_VAR_NAME, [])
+        return scope
 
 
 # Backend loading was previously performed via the
