@@ -4,6 +4,7 @@ from social_auth.utils import setting
 from social_auth.models import UserSocialAuth
 from social_auth.backends.pipeline import warn_setting
 from social_auth.backends.exceptions import AuthException
+from django.utils.translation import ugettext
 
 
 def social_auth_user(backend, uid, user=None, *args, **kwargs):
@@ -12,16 +13,11 @@ def social_auth_user(backend, uid, user=None, *args, **kwargs):
 
     Raise AuthException if UserSocialAuth entry belongs to another user.
     """
-    try:
-        social_user = UserSocialAuth.objects.select_related('user')\
-                                            .get(provider=backend.name,
-                                                 uid=uid)
-    except UserSocialAuth.DoesNotExist:
-        social_user = None
-
+    social_user = UserSocialAuth.get_social_auth(backend.name, uid)
     if social_user:
         if user and social_user.user != user:
-            raise AuthException(backend, 'Account already in use.')
+            msg = ugettext('This %(provider)s account already in use.')
+            raise AuthException(backend, msg % {'provider': backend.name})
         elif not user:
             user = social_user.user
     return {'social_user': social_user, 'user': user}
@@ -33,8 +29,7 @@ def associate_user(backend, user, uid, social_user=None, *args, **kwargs):
         return None
 
     try:
-        social = UserSocialAuth.objects.create(user=user, uid=uid,
-                                               provider=backend.name)
+        social = UserSocialAuth.create_social_auth(user, uid, backend.name)
     except IntegrityError:
         # Protect for possible race condition, those bastard with FTL
         # clicking capabilities, check issue #131:
@@ -55,5 +50,8 @@ def load_extra_data(backend, details, response, social_user, uid, user,
     if setting('SOCIAL_AUTH_EXTRA_DATA', True):
         extra_data = backend.extra_data(user, uid, response, details)
         if extra_data and social_user.extra_data != extra_data:
-            social_user.extra_data = extra_data
+            if social_user.extra_data:
+                social_user.extra_data.update(extra_data)
+            else:
+                social_user.extra_data = extra_data
             social_user.save()
