@@ -103,6 +103,8 @@ def complete_process(request, backend, *args, **kwargs):
 
     if user:
         if getattr(user, 'is_active', True):
+            # catch is_new flag before login() might reset the instance
+            is_new = getattr(user, 'is_new', False)
             login(request, user)
             # user.social_user is the used UserSocialAuth instance defined
             # in authenticate process
@@ -115,8 +117,13 @@ def complete_process(request, backend, *args, **kwargs):
                 # Set session expiration date if present and not disabled by
                 # setting. Use last social-auth instance for current provider,
                 # users can associate several accounts with a same provider.
-                if social_user.expiration_delta():
-                    request.session.set_expiry(social_user.expiration_delta())
+                expiration = social_user.expiration_datetime()
+                if expiration:
+                    try:
+                        request.session.set_expiry(expiration)
+                    except OverflowError:
+                        # Handle django time zone overflow, set default expiry.
+                        request.session.set_expiry(None)
 
             # store last login backend name in session
             key = setting('SOCIAL_AUTH_LAST_LOGIN',
@@ -127,7 +134,7 @@ def complete_process(request, backend, *args, **kwargs):
             # account, send him to the new-users-page if defined.
             new_user_redirect = backend_setting(backend,
                                            'SOCIAL_AUTH_NEW_USER_REDIRECT_URL')
-            if new_user_redirect and getattr(user, 'is_new', False):
+            if new_user_redirect and is_new:
                 url = new_user_redirect
             else:
                 url = redirect_value or \
